package com.qmth.teachcloud.cas.api;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.teachcloud.cas.util.CdutCasUtil;
import com.qmth.teachcloud.cas.util.GdpuCasUtil;
import com.qmth.teachcloud.cas.util.SxufeCasUtil;
import com.qmth.teachcloud.cas.util.XjtuCasUtil;
import com.qmth.teachcloud.cas.util.supwisdom.CasUtils;
import com.qmth.teachcloud.cas.util.sxufe.Constants;
import com.qmth.teachcloud.exchange.common.bean.params.RedirectParams;
import com.qmth.teachcloud.exchange.common.contant.SystemConstant;
import com.qmth.teachcloud.exchange.common.enums.SchoolCodeEnum;
import com.qmth.teachcloud.exchange.common.service.CommonService;
import com.qmth.teachcloud.exchange.common.util.JacksonUtil;
import com.qmth.teachcloud.exchange.common.util.Result;
import com.qmth.teachcloud.exchange.common.util.ServletUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

/**
 * <p>
 * cas开放接口前端控制器
 * </p>
 *
 * @author wangliang
 * @since 2022-04-26
 */
@Api(tags = "cas开放接口Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + SystemConstant.PREFIX_URL_OPEN)
@Validated
public class OpenApiController {
    private static final Logger log = LoggerFactory.getLogger(OpenApiController.class);
    private static final String ACCESS_TOKEN_URL = "http://org.xjtu.edu.cn/openplatform/oauth/getAccessToken";
    private static final String USER_INFO_URL = "http://org.xjtu.edu.cn/openplatform/oauth/open/getUserInfo";
    private static final String LOGOUT_URL = "http://org.xjtu.edu.cn/openplatform/oauth/logout";
    //    private static final String SCHOOL_CODE = "xjtu";//测试学校code，正式改成xjtu
    private static final String LOGIN_BEFORE_XJU_LOGIC_API = "/api/admin/print/open/login_before_xju_logic";//西交大登录之前逻辑
//    private static final String VERSION = "1.0.1.1";

    @Resource
    CommonService commonService;

//    @Value("${cas.config.logoutUrl}")
//    String logoutUrl;
//
//    @Value("${cas.config.returnUrl}")
//    String returnUrl;
//
//    @Value("${cas.config.teachcloudPrintLoginUrl}")
//    String teachcloudLoginUrl;

    @Resource
    XjtuCasUtil xjtuCasUtil;

    @Resource
    CdutCasUtil cdutCasUtil;

    @Resource
    SxufeCasUtil sxufeCasUtil;

    @Resource
    GdpuCasUtil gdpuCasUtil;

    @ApiOperation(value = "cas鉴权接口")
    @ApiResponses({@ApiResponse(code = 200, message = "返回消息", response = Result.class)})
    @RequestMapping(value = "/{schoolCode}/authentication", method = RequestMethod.GET)
    @Aac(auth = BOOL.FALSE)
    public void authentication(@ApiParam(value = "学校编码") @PathVariable("schoolCode") String schoolCode,
                               @ApiParam(value = "工号") @RequestParam(required = false) String code,
                               @ApiParam(value = "系统参数") @RequestParam(required = false) String state,
                               @ApiParam(value = "用户类型") @RequestParam(required = false) String userType,
                               @ApiParam(value = "员工工号") @RequestParam(required = false) String employeeNo,
                               @ApiParam(value = "返回url") @RequestParam(required = false) String returnUrl) throws IOException {
        RedirectParams redirectParams = null;
        switch (SchoolCodeEnum.valueOf(schoolCode.toUpperCase())) {
            case XJTU:
                redirectParams = xjtuCasUtil.authentication(schoolCode, code, state, userType, employeeNo, returnUrl);
                break;
            case CDUT:
                cdutCasUtil.authentication(schoolCode);
                break;
            case SXUFE:
                sxufeCasUtil.authentication(schoolCode);
                break;
            case GDPU:
                gdpuCasUtil.authentication(schoolCode);
                break;
            default:
                break;
        }
        if (Objects.nonNull(redirectParams)) {
            commonService.redirectLogic(redirectParams.getEmployeeNo(), redirectParams.getSchoolCode(), redirectParams.getReturnUrl(), Objects.nonNull(redirectParams.getOpenParams()) ? JacksonUtil.parseJson(redirectParams.getOpenParams()) : null);
        }
    }

    @ApiOperation(value = "cas鉴权退出接口")
    @RequestMapping(value = "/{schoolCode}/authentication/logout", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "返回消息", response = Result.class)})
    @Aac(auth = BOOL.FALSE)
    public void logout(@ApiParam(value = "学校编码") @PathVariable("schoolCode") String schoolCode,
                       @ApiParam(value = "sessionId", required = true) @RequestParam String gSessionId) throws IOException {
        switch (SchoolCodeEnum.valueOf(schoolCode.toUpperCase())) {
            case XJTU:
                xjtuCasUtil.logout(schoolCode, gSessionId);
                break;
            case CDUT:
                cdutCasUtil.logout(schoolCode);
                break;
            case SXUFE:
                sxufeCasUtil.logout(schoolCode);
                break;
            case GDPU:
                gdpuCasUtil.logout(schoolCode);
                break;
            default:
                break;
        }
    }

    @ApiOperation(value = "跳转知学知考")
    @RequestMapping(value = "/{schoolCode}/authentication/zxzk_login", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "返回消息", response = Result.class)})
    @Aac(auth = BOOL.FALSE)
    public void zxzkLogin(@ApiParam(value = "学校编码") @PathVariable("schoolCode") String schoolCode) throws IOException {
        HttpServletRequest request = ServletUtil.getRequest();
        HttpServletResponse response = ServletUtil.getResponse();
        RedirectParams redirectParams = null;
        switch (SchoolCodeEnum.valueOf(schoolCode.toUpperCase())) {
            case CDUT:
                redirectParams = cdutCasUtil.zxzkLogin(schoolCode);
                if (Objects.nonNull(redirectParams)) {
                    commonService.redirectLogic(redirectParams.getEmployeeNo(), redirectParams.getSchoolCode(), redirectParams.getReturnUrl(), Objects.nonNull(redirectParams.getOpenParams()) ? JacksonUtil.parseJson(redirectParams.getOpenParams()) : null);
                } else {
                    String loginUrl = CasUtils.getLoginUrl(request);
                    response.sendRedirect(loginUrl);
                }
                break;
            case SXUFE:
                redirectParams = sxufeCasUtil.zxzkLogin(schoolCode);
                if (Objects.nonNull(redirectParams)) {
                    commonService.redirectLogic(redirectParams.getEmployeeNo(), redirectParams.getSchoolCode(), redirectParams.getReturnUrl(), Objects.nonNull(redirectParams.getOpenParams()) ? JacksonUtil.parseJson(redirectParams.getOpenParams()) : null);
                } else {
                    response.setHeader("Access-Control-Allow-Origin", SystemConstant.PATH_MATCH);
                    response.sendRedirect(Constants.CAS_LOGIN_URL);
                }
                break;
            case GDPU:
                redirectParams = gdpuCasUtil.zxzkLogin(schoolCode);
                if (Objects.nonNull(redirectParams)) {
                    commonService.redirectLogic(redirectParams.getEmployeeNo(), redirectParams.getSchoolCode(), redirectParams.getReturnUrl(), Objects.nonNull(redirectParams.getOpenParams()) ? JacksonUtil.parseJson(redirectParams.getOpenParams()) : null);
                } else {
                    response.setHeader("Access-Control-Allow-Origin", SystemConstant.PATH_MATCH);
                    response.sendRedirect(com.qmth.teachcloud.cas.util.gdpu.Constants.CAS_LOGIN_URL);
                }
                break;
            default:
                break;
        }
    }
}
